@@ -6,13 +6,17 @@
  * This module is Promises friendly where needed
  */
 
+if (typeof msgtools !== "undefined") {
+    console.log('msgtools already loaded')
+} else {
 
  var msgtools = (function() {
     // Used to store a dictionary of all messages - key is the msg ID,
     // and the value is a reference to the message class itself
-    var MessageDictionary = new Map()
+    var MessageDictionary = new Map();
     // same, but mapping from name to message class.
-    var MessageNameDictionary = new Map()
+    var MessageNameDictionary = new Map();
+    var MessageClassTree = Object.create(null);
 
     // Base messasge directory we load generated messages from
     // You must call load to initialize this method and load all
@@ -127,7 +131,22 @@
      */
     function registerMessage(id, msgClass) {
         MessageDictionary.set(id, msgClass);
-        MessageNameDictionary.set(msgClass.name, msgClass);
+        var name = msgClass.prototype.MsgName();
+        MessageNameDictionary.set(name, msgClass);
+        
+        var nested = MessageClassTree;
+        var parts = name.split(".");
+        for(var i=0; i<parts.length; i++) {
+            part = parts[i];
+            if(!(part in nested)) {
+                if(i < parts.length-1) {
+                    nested[part] = Object.create(null);
+                } else {
+                    nested[part] = msgClass;
+                }
+            }
+            nested = nested[part];
+        }
     }
 
     /**
@@ -171,18 +190,36 @@
     
     class DelayedInit {
         static add(w) {
-            DelayedInit.widgets.push(w);
+            if(DelayedInit.alreadyInitialized) {
+                //console.log("    DelayedInit: added without delay");
+                w.init();
+            } else {
+                //console.log("    DelayedInit.add() NOT adding to list");
+                DelayedInit.widgets.push(w);
+            }
         }
         static init() {
             while(DelayedInit.widgets.length > 0) {
                 DelayedInit.widgets.pop().init();
             }
+            DelayedInit.alreadyInitialized = true;
         }
     }
+    //when running in a grafana panel, msgtools gets loaded once and this array is initialized to empty,
+    //but widgets are put into it again and again when the Text panel HTML content is edited!
+    //maybe this needs to be a hash table with a static key, so each thing is only put in once?
+    //or maybe widgets need to remove themselves when they are destroyed?
+    //how, though?  javascript has no concept of destructor!?!?!
     DelayedInit.widgets = [];
+    DelayedInit.alreadyInitialized = false;
     class MessageDispatch {
         constructor() {
+            // a lookup table of a list of listeners for each message ID
             this.m_listeners = {};
+            // a cache of the last received message of each ID.
+            // used to give new listeners the last received message
+            // when they are created
+            this.m_rxCache = {};
         }
         
         register(id, handler) {
@@ -190,17 +227,22 @@
                 this.m_listeners[id] = [];
             }
             this.m_listeners[id].push(handler);
+            if(id in this.m_rxCache) {
+                handler(this.m_rxCache[id]);
+            }
         }
 
         remove(id, handler) {
-            var listeners = this.m_listeners[id];
+            let listeners = this.m_listeners[id];
             if (listeners !== undefined) {
                 listeners.delete(handler);
             }
         }
         
         deliver(msg) {
-            var listeners = this.m_listeners[msg.hdr.GetMessageID()];
+            const id = msg.hdr.GetMessageID();
+            let listeners = this.m_listeners[id];
+            this.m_rxCache[id] = msg;
             if (listeners !== undefined) {
                 for( let l of listeners ) {
                     l(msg);
@@ -570,6 +612,11 @@
         DelayedInit : DelayedInit,
         UnknownMsg : UnknownMsg,
         toJSON : toJSON,
-        getWebsocketURLParams : getWebsocketURLParams
+        getWebsocketURLParams : getWebsocketURLParams,
+        messageNameDictionary : MessageNameDictionary,
+        msgs  : MessageClassTree
     }
-})()
\ No newline at end of file
+})()
+
+window.msgtools = msgtools;
+}
